@@ -1,8 +1,8 @@
-/*$Id: fkHttpChannel.cpp,v 1.13 2007/03/14 16:17:29 jwrobel Exp $*/
+/*$Id: fkHttpChannel.cpp,v 1.17 2007/07/22 21:03:11 jwrobel Exp $*/
 /* ***** BEGIN LICENSE BLOCK *****
  *  This file is part of Firekeeper.
  *
- *  Copyright (C) 2006 Jan Wrobel <wrobel@blues.ath.cx>
+ *  Copyright (C) 2006,2007 Jan Wrobel <wrobel@blues.ath.cx>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  * ***** END LICENSE BLOCK ***** */
 
+
 #include "nsCOMPtr.h"
 #include "nsXPCOM.h"
 #include "nsISupportsImpl.h"
@@ -26,9 +27,9 @@
 #include "nsEmbedString.h"
 
 #include "fkRule.h"
-#include "fkHttpChannel.h"
 #include "fkStreamListener.h"
 #include "Firekeeper.h"
+#include "Error.h"
 
 /*
  * Create new fkHttpChannel
@@ -43,7 +44,7 @@
 fkHttpChannel::fkHttpChannel(nsIChannel *origChannel, fkISentence *sentence, 
 			     ActiveRuleSet *connRules)
 {
-	TRACE("constructor %08x", this);
+	TRACE("constructor %08x %08x", this, origChannel);
 	nsresult rv;
 	
 	this->origChannel = origChannel;
@@ -52,7 +53,7 @@ fkHttpChannel::fkHttpChannel(nsIChannel *origChannel, fkISentence *sentence,
 	
 	origRequest = do_QueryInterface(origChannel, &rv); 
 	checkResult(rv, "nsIRequest");
-	
+
 	origHttpChannel = do_QueryInterface(origChannel, &rv); 
 	checkResult(rv, "nsIHttpChannel");
 
@@ -100,17 +101,122 @@ fkHttpChannel::fkHttpChannel(nsIChannel *origChannel, fkISentence *sentence,
 
 	origWritablePropertyBag2 = do_QueryInterface(origChannel, &rv); 
 	checkResult(rv, "nsIWritablePropertyBag2");	
+
+	callbacksProxy = nsnull;
+	loadGroupProxy = nsnull;
+	
+#ifdef DEBUG
+	nsCOMPtr<nsILoadGroup> lg;
+	rv = origRequest->GetLoadGroup(getter_AddRefs(lg));
+	FK_ASSERT(!lg);
+
+	nsCOMPtr<nsIInterfaceRequestor> cb;	
+	rv = origChannel->GetNotificationCallbacks(getter_AddRefs(cb));
+	FK_ASSERT(!cb);
+#endif	
+
 }
 
 fkHttpChannel::~fkHttpChannel()
 {
 	TRACE("destructor %08x", this);
+
+#ifdef DEBUG
+	FK_ASSERT(requestHashTable->Get(origHttpChannel, nsnull));
+#endif	
+	requestHashTable->Remove(origHttpChannel);
+	TRACE("HashTable count %d", requestHashTable->Count());
+	if (this->connRules)
+		delete connRules;
+}
+
+NS_IMETHODIMP 
+fkHttpChannel::init()
+{
+	return NS_OK;
 }
 
 NS_IMPL_ADDREF(fkHttpChannel)
 NS_IMPL_RELEASE(fkHttpChannel)
 
+#if 1
+/*Only interfaces implemented by origLoadGroup are returned.*/
+NS_IMETHODIMP
+fkHttpChannel::QueryInterface(const nsIID &aIID, void **aResult)
+{	
+	if (aResult == NULL) {
+		return NS_ERROR_NULL_POINTER;
+	}
+	nsISupports *foundInterface = 0;
+	*aResult = NULL;
 
+	if (aIID.Equals(NS_GET_IID(nsIRequest))){
+		foundInterface = NS_STATIC_CAST(nsIRequest*, this);
+	}else if (aIID.Equals(NS_GET_IID(nsIChannel))){
+		foundInterface = NS_STATIC_CAST(nsIChannel*, this);
+	}else if (aIID.Equals(NS_GET_IID(nsIRequestObserver))){
+		foundInterface = NS_STATIC_CAST(nsIRequestObserver*, this);
+	}else if (aIID.Equals(NS_GET_IID(nsIStreamListener))){
+		foundInterface = NS_STATIC_CAST(nsIStreamListener*, this);
+	}else if (aIID.Equals(NS_GET_IID(nsIHttpChannel))){
+		foundInterface = NS_STATIC_CAST(nsIHttpChannel*, this);
+	}else if (aIID.Equals(NS_GET_IID(nsICachingChannel))){
+		foundInterface = NS_STATIC_CAST(nsICachingChannel*, this);
+	}else if (aIID.Equals(NS_GET_IID(nsIUploadChannel))){
+		foundInterface = NS_STATIC_CAST(nsIUploadChannel*, this);
+	}else if (aIID.Equals(NS_GET_IID(nsICacheListener))){
+		foundInterface = NS_STATIC_CAST(nsICacheListener*, this);
+	}else if (aIID.Equals(NS_GET_IID(nsIEncodedChannel))){
+		foundInterface = NS_STATIC_CAST(nsIEncodedChannel*, this);
+	}else if (aIID.Equals(NS_GET_IID(nsIHttpChannelInternal))){
+		foundInterface = NS_STATIC_CAST(nsIHttpChannelInternal*, this);
+	}else if (aIID.Equals(NS_GET_IID(nsIResumableChannel))){
+		foundInterface = NS_STATIC_CAST(nsIResumableChannel*, this);
+	}else if (aIID.Equals(NS_GET_IID(nsITransportEventSink))){
+		foundInterface = NS_STATIC_CAST(nsITransportEventSink*, this);
+	}else if (aIID.Equals(NS_GET_IID(nsISupportsPriority))){
+		foundInterface = NS_STATIC_CAST(nsISupportsPriority*, this);
+	}else if (aIID.Equals(NS_GET_IID(nsIProtocolProxyCallback))){
+		foundInterface = NS_STATIC_CAST(nsIProtocolProxyCallback*, this);
+	}else if (aIID.Equals(NS_GET_IID(nsIWritablePropertyBag))){
+		foundInterface = NS_STATIC_CAST(nsIWritablePropertyBag*, this);
+	}else if (aIID.Equals(NS_GET_IID(nsIPropertyBag))){
+		foundInterface = NS_STATIC_CAST(nsIPropertyBag*, NS_STATIC_CAST(nsIWritablePropertyBag*, this));
+	}else if (aIID.Equals(NS_GET_IID(nsISupports))){
+		foundInterface = NS_STATIC_CAST(nsISupports*, NS_STATIC_CAST(nsIWritablePropertyBag*, this));
+	}else if (aIID.Equals(NS_GET_IID(nsIPropertyBag2))){
+		foundInterface = NS_STATIC_CAST(nsIPropertyBag2*, this);
+	}else if (aIID.Equals(NS_GET_IID(nsIWritablePropertyBag2))){
+		foundInterface = NS_STATIC_CAST(nsIWritablePropertyBag2*, this);
+	}
+
+	
+	if (!foundInterface) {
+		TRACE("fkHttpChannel: interface not implemented %s .", aIID.ToString());
+#ifdef DEBUG
+		if (origChannel){
+			void *dummy = nsnull;
+			/*this code trigers assertion when aIID is nsIInterfaceRequestor (?)*/
+
+			/*This leaks memory but only in debug mode when FK_ASSERT will be
+			  triggered anyway*/
+			nsresult rv = origChannel->QueryInterface(aIID, &dummy);			
+			if (!NS_FAILED(rv) && dummy){
+				TRACE("origChannel implements this!!!!");				
+				FK_ASSERT(dummy);
+			}
+		}
+#endif
+		return NS_NOINTERFACE;
+	}
+
+	TRACE("interface implemented %s .", aIID.ToString());
+	NS_ADDREF(foundInterface);
+	*aResult = foundInterface;
+	return NS_OK;
+} 
+
+#else
 NS_INTERFACE_MAP_BEGIN(fkHttpChannel)
     NS_INTERFACE_MAP_ENTRY(nsIRequest)
     NS_INTERFACE_MAP_ENTRY(nsIChannel)
@@ -134,7 +240,7 @@ NS_INTERFACE_MAP_BEGIN(fkHttpChannel)
     NS_INTERFACE_MAP_ENTRY(nsIWritablePropertyBag2)
 NS_INTERFACE_MAP_END
 
-
+#endif
 //-----------------------------------------------------------------------------
 // fkHttpChannel::nsIRequest
 //-----------------------------------------------------------------------------
@@ -142,12 +248,14 @@ NS_INTERFACE_MAP_END
 NS_IMETHODIMP
 fkHttpChannel::GetName(nsACString &aName)
 {
+	TRACE("");
 	return origRequest->GetName(aName);
 }
 
 NS_IMETHODIMP
 fkHttpChannel::IsPending(PRBool *value)
 {
+	TRACE("");
 	return origRequest->IsPending(value);
 }
 
@@ -161,43 +269,107 @@ fkHttpChannel::GetStatus(nsresult *aStatus)
 NS_IMETHODIMP
 fkHttpChannel::Cancel(nsresult status)
 {
-	return origRequest->Cancel(status);
+	TRACE("");
+	nsresult rv = origRequest->Cancel(status);
+	if (NS_FAILED(rv)){
+		TRACE("cancel failed");
+	}
+	return rv;
 }
 
 NS_IMETHODIMP
 fkHttpChannel::Suspend()
 {
+	TRACE("");
 	return origRequest->Suspend();
 }
 
 NS_IMETHODIMP
 fkHttpChannel::Resume()
 {
+	TRACE("");
 	return origRequest->Resume();
 }
 
 NS_IMETHODIMP
 fkHttpChannel::GetLoadGroup(nsILoadGroup **aLoadGroup)
-{
-	return origRequest->GetLoadGroup(aLoadGroup);
+{	
+	TRACE("fkHttpChannel");
+	NS_ENSURE_ARG_POINTER(aLoadGroup);
+	if (!loadGroupProxy){
+		*aLoadGroup = nsnull;
+		return NS_OK;
+	}else{
+		nsresult rv;
+		nsCOMPtr<fkILoadGroupProxy> p = do_QueryInterface(loadGroupProxy, &rv);
+		FK_ASSERT(!NS_FAILED(rv));
+		return p->GetOrigLoadGroup(aLoadGroup);
+	}
 }
 
 NS_IMETHODIMP
 fkHttpChannel::SetLoadGroup(nsILoadGroup *aLoadGroup)
 {
-	return origRequest->SetLoadGroup(aLoadGroup);
+	nsresult rv;
+	TRACE("setting load group %08x origChannel = %08x this = %08x", 
+	      aLoadGroup, &*origChannel, this);	
+	loadGroupProxy = nsnull;
+	
+	if (aLoadGroup){
+		/*If this cast succeeds aLoadGroup is already a proxy (instance of fkLoadGroup).
+		  We should get and use original LoadGroup from this proxy to avoid 
+		  call chains like this:		  
+		  proxy1->OnStartRequest(nsRequest...) calls proxy2->OnStartRequest(fkReqeuest1)
+		  calls origObserver->OnStartReqeust(fkRequest2) (This is wrong, fkReqeust1 has 
+		  started not fkRequest2).
+		*/
+		nsCOMPtr<fkILoadGroupProxy> tmp = do_QueryInterface(aLoadGroup, &rv);
+		if (!NS_FAILED(rv)){
+			TRACE("proxy already used");
+			loadGroupProxy = aLoadGroup;
+		}
+		else{
+			fkLoadGroup *proxy;
+			if (!groupHashTable->Get(aLoadGroup, &proxy)){
+				TRACE("group not found");
+				proxy = new fkLoadGroup(aLoadGroup);		
+				if (!proxy || NS_FAILED(proxy->init()))
+					return NS_ERROR_OUT_OF_MEMORY;
+				groupHashTable->Put(aLoadGroup, proxy);
+
+			}
+			else{
+				TRACE("group found");
+			}
+			loadGroupProxy = proxy;
+		}
+	}
+	return origChannel->SetLoadGroup(loadGroupProxy);
+
 }
 
 NS_IMETHODIMP
 fkHttpChannel::GetLoadFlags(nsLoadFlags *aLoadFlags)
 {
-	return origRequest->GetLoadFlags(aLoadFlags);
+	nsresult rv = origRequest->GetLoadFlags(aLoadFlags);
+	TRACE("flags = %08x", *aLoadFlags);
+	if (*aLoadFlags & nsIChannel::LOAD_REPLACE){
+		TRACE("fkHttpChannel REPLACE FLAG SET");
+	}
+	FK_ASSERT(!NS_FAILED(rv));
+	return rv;
 }
 
 NS_IMETHODIMP
 fkHttpChannel::SetLoadFlags(nsLoadFlags aLoadFlags)
 {
-	return origRequest->SetLoadFlags(aLoadFlags);
+	nsresult rv = origRequest->SetLoadFlags(aLoadFlags);
+	TRACE("fkHttpChannel flags = %08x", aLoadFlags);
+	if (aLoadFlags & nsIChannel::LOAD_REPLACE){
+		TRACE("fkHttpChannel REPLACE FLAG SET");
+	}
+	FK_ASSERT(!NS_FAILED(rv));
+	return rv;
 }
 
 //-----------------------------------------------------------------------------
@@ -207,54 +379,99 @@ fkHttpChannel::SetLoadFlags(nsLoadFlags aLoadFlags)
 NS_IMETHODIMP
 fkHttpChannel::GetOriginalURI(nsIURI **originalURI)
 {
+	TRACE("");
 	return origChannel->GetOriginalURI(originalURI);
 }
 
 NS_IMETHODIMP
 fkHttpChannel::SetOriginalURI(nsIURI *originalURI)
 {
+	TRACE("");
 	return origChannel->SetOriginalURI(originalURI);
 }
 
 NS_IMETHODIMP
 fkHttpChannel::GetURI(nsIURI **URI)
 {
+	TRACE("");
 	return origChannel->GetURI(URI);
 }
 
 NS_IMETHODIMP
 fkHttpChannel::GetOwner(nsISupports **owner)
 {
+	TRACE("");
 	return origChannel->GetOwner(owner);
 }
 
 NS_IMETHODIMP
 fkHttpChannel::SetOwner(nsISupports *owner)
 {
+	TRACE("");
 	return origChannel->SetOwner(owner);
 }
 
 NS_IMETHODIMP
 fkHttpChannel::GetNotificationCallbacks(nsIInterfaceRequestor **callbacks)
-{
-	return origChannel->GetNotificationCallbacks(callbacks);
+{	
+	
+	TRACE("fkHttpChannel");
+	NS_ENSURE_ARG_POINTER(callbacks);
+	if (!callbacksProxy){
+		*callbacks = nsnull;
+		return NS_OK;
+	}
+	
+	nsresult rv;
+	nsCOMPtr<fkICallbacksProxy> p = do_QueryInterface(callbacksProxy, &rv);
+	FK_ASSERT(!NS_FAILED(rv));
+	return p->GetOrigCallbacks(callbacks);
 }
 
 NS_IMETHODIMP
 fkHttpChannel::SetNotificationCallbacks(nsIInterfaceRequestor *callbacks)
 {
-	return origChannel->SetNotificationCallbacks(callbacks);
+	nsresult rv;
+	TRACE("Setting callbacks %08x origChannel = %08x this = %08x", 
+	      callbacks, &*origChannel, this);	
+	callbacksProxy = nsnull;
+	
+	if (callbacks){	
+		nsCOMPtr<fkICallbacksProxy> tmp = do_QueryInterface(callbacks, &rv);
+		if (!NS_FAILED(rv)){
+			TRACE("proxy already used");
+			callbacksProxy = callbacks;
+		}else{
+			fkCallbacks *proxy;
+			if (!callbacksHashTable->Get(callbacks, &proxy)){
+				TRACE("callbacks not found");
+				proxy = new fkCallbacks(nsnull, callbacks);		
+				if (!proxy)
+					return NS_ERROR_OUT_OF_MEMORY;
+				callbacksHashTable->Put(callbacks, proxy);
+			}
+			else{
+				TRACE("callbacks found");
+			}
+			callbacksProxy = proxy;
+		}
+	}
+	return origChannel->SetNotificationCallbacks(callbacksProxy);
 }
+	
+
 
 NS_IMETHODIMP
 fkHttpChannel::GetSecurityInfo(nsISupports **securityInfo)
 {
+	TRACE("");
 	return origChannel->GetSecurityInfo(securityInfo);
 }
 
 NS_IMETHODIMP
 fkHttpChannel::GetContentType(nsACString &value)
 {
+	TRACE("");
 	return origChannel->GetContentType(value);
 #if 0	/* 
 	   Remains of attempts to replace blocked traffic.
@@ -281,30 +498,35 @@ fkHttpChannel::GetContentType(nsACString &value)
 NS_IMETHODIMP
 fkHttpChannel::SetContentType(const nsACString &value)
 {
+	TRACE("");
 	return origChannel->SetContentType(value);
 }
 
 NS_IMETHODIMP
 fkHttpChannel::GetContentCharset(nsACString &value)
 {
+	TRACE("");
 	return origChannel->GetContentCharset(value);
 }
 
 NS_IMETHODIMP
 fkHttpChannel::SetContentCharset(const nsACString &value)
 {
+	TRACE("");
 	return origChannel->SetContentCharset(value);
 }
 
 NS_IMETHODIMP
 fkHttpChannel::GetContentLength(PRInt32 *value)
 {
+	TRACE("");
 	return origChannel->GetContentLength(value);
 }
 
 NS_IMETHODIMP
 fkHttpChannel::SetContentLength(PRInt32 value)
 {
+	TRACE("");
 	return origChannel->SetContentLength(value);
 }
 
@@ -323,15 +545,32 @@ fkHttpChannel::Open(nsIInputStream **_retval)
 NS_IMETHODIMP
 fkHttpChannel::AsyncOpen(nsIStreamListener *listener, nsISupports *context)
 {	
-	TRACE("");
-	
 	int action;
+	nsresult rv;
+	
+	TRACE("");
+	NS_ENSURE_ARG_POINTER(listener);
+	
 	sentence->GetAction(&action);
 	if (action == sentence->BLOCK)
 		return NS_ERROR_ABORT;
-	if (action == sentence->DONT_AUDIT)		
+	if (action == sentence->DONT_AUDIT){		
+		/*TODO: can't happen*/
+		TRACE("whitelisted");
+		FK_ASSERT(false);
 		return origChannel->AsyncOpen(listener, context);
+	}
 
+	nsCOMPtr<fkIStreamListenerProxy> tmp = do_QueryInterface(listener, &rv);
+	if (!NS_FAILED(rv)){
+		TRACE("listener is already a proxy");
+		/* This happens when channel is redirected. StreamListener from
+		   old channel is used in AsyncOpen Call */
+		return origChannel->AsyncOpen(listener, context);
+	}
+	
+	TRACE("checking");
+	
 	fkStreamListener *fklistener_raw =
 		new fkStreamListener(listener, this, sentence, connRules);
 	
@@ -339,9 +578,8 @@ fkHttpChannel::AsyncOpen(nsIStreamListener *listener, nsISupports *context)
 		return NS_ERROR_OUT_OF_MEMORY;
 
 	nsCOMPtr<nsIStreamListener> fklistener;
-	nsresult rv = fklistener_raw->
-		QueryInterface(NS_GET_IID(nsIStreamListener), 
-			       getter_AddRefs(fklistener));
+	rv = fklistener_raw->QueryInterface(NS_GET_IID(nsIStreamListener), 
+					    getter_AddRefs(fklistener));
 	if (NS_FAILED(rv))
 		return rv;
 	
@@ -357,30 +595,35 @@ fkHttpChannel::AsyncOpen(nsIStreamListener *listener, nsISupports *context)
 NS_IMETHODIMP
 fkHttpChannel::GetRequestMethod(nsACString &method)
 {
+	TRACE("");
 	return origHttpChannel->GetRequestMethod(method);
 }
 
 NS_IMETHODIMP
 fkHttpChannel::SetRequestMethod(const nsACString &method)
 {
+	TRACE("");
 	return origHttpChannel->SetRequestMethod(method);
 }
 
 NS_IMETHODIMP
 fkHttpChannel::GetReferrer(nsIURI **referrer)
 {
+	TRACE("");
 	return origHttpChannel->GetReferrer(referrer);
 }
 
 NS_IMETHODIMP
 fkHttpChannel::SetReferrer(nsIURI *referrer)
 {
+	TRACE("");
 	return origHttpChannel->SetReferrer(referrer);
 }
 
 NS_IMETHODIMP
 fkHttpChannel::GetRequestHeader(const nsACString &header, nsACString &value)
 {
+	TRACE("");
 	return origHttpChannel->GetRequestHeader(header, value);
 }
 
@@ -389,12 +632,14 @@ fkHttpChannel::SetRequestHeader(const nsACString &header,
                                 const nsACString &value,
                                 PRBool merge)
 {
+	TRACE("");
 	return origHttpChannel->SetRequestHeader(header, value, merge);
 }
 
 NS_IMETHODIMP
 fkHttpChannel::VisitRequestHeaders(nsIHttpHeaderVisitor *visitor)
 {
+	TRACE("");
 	return origHttpChannel->VisitRequestHeaders(visitor);
 }
 
@@ -402,24 +647,28 @@ fkHttpChannel::VisitRequestHeaders(nsIHttpHeaderVisitor *visitor)
 NS_IMETHODIMP
 fkHttpChannel::GetResponseStatus(PRUint32 *value)
 {
+	TRACE("");
 	return origHttpChannel->GetResponseStatus(value);
 }
 
 NS_IMETHODIMP
 fkHttpChannel::GetResponseStatusText(nsACString &value)
 {
+	TRACE("");
 	return origHttpChannel->GetResponseStatusText(value);
 }
 
 NS_IMETHODIMP
 fkHttpChannel::GetRequestSucceeded(PRBool *value)
 {
+	TRACE("");
 	return origHttpChannel->GetRequestSucceeded(value);
 }
 
 NS_IMETHODIMP
 fkHttpChannel::GetResponseHeader(const nsACString &header, nsACString &value)
 {
+	TRACE("");
 	return origHttpChannel->GetResponseHeader(header, value);
 }
 
@@ -428,48 +677,56 @@ fkHttpChannel::SetResponseHeader(const nsACString &header,
                                  const nsACString &value,
                                  PRBool merge)
 {
+	TRACE("");
 	return origHttpChannel->SetResponseHeader(header, value, merge);
 }
 
 NS_IMETHODIMP
 fkHttpChannel::VisitResponseHeaders(nsIHttpHeaderVisitor *visitor)
 {
+	TRACE("");
 	return origHttpChannel->VisitResponseHeaders(visitor);
 }
 
 NS_IMETHODIMP
 fkHttpChannel::IsNoStoreResponse(PRBool *value)
 {
+	TRACE("");
 	return origHttpChannel->IsNoStoreResponse(value);
 }
 
 NS_IMETHODIMP
 fkHttpChannel::IsNoCacheResponse(PRBool *value)
 {
+	TRACE("");
 	return origHttpChannel->IsNoCacheResponse(value);
 }
 
 NS_IMETHODIMP
 fkHttpChannel::GetAllowPipelining(PRBool *value)
 {
+	TRACE("");
 	return origHttpChannel->GetAllowPipelining(value);
 }
 
 NS_IMETHODIMP
 fkHttpChannel::SetAllowPipelining(PRBool value)
 {
+	TRACE("");
 	return origHttpChannel->SetAllowPipelining(value);
 }
 
 NS_IMETHODIMP
 fkHttpChannel::GetRedirectionLimit(PRUint32 *value)
 {
+	TRACE("");
 	return origHttpChannel->GetRedirectionLimit(value);
 }
 
 NS_IMETHODIMP
 fkHttpChannel::SetRedirectionLimit(PRUint32 value)
 {
+	TRACE("");
 	return origHttpChannel->SetRedirectionLimit(value);
 }
 
@@ -502,6 +759,7 @@ fkHttpChannel::SetApplyConversion(PRBool value)
 NS_IMETHODIMP
 fkHttpChannel::GetUploadStream(nsIInputStream **stream)
 {
+	TRACE("");
 	return origUploadChannel->GetUploadStream(stream);
 }
 
@@ -511,6 +769,7 @@ NS_IMETHODIMP
 fkHttpChannel::SetUploadStream(nsIInputStream *stream, 
 			       const nsACString &contentType, PRInt32 contentLength)
 {
+	TRACE("");
 	return origUploadChannel->SetUploadStream(stream, contentType, contentLength);
 }
 
@@ -521,12 +780,14 @@ fkHttpChannel::SetUploadStream(nsIInputStream *stream,
 NS_IMETHODIMP
 fkHttpChannel::GetDocumentURI(nsIURI **aDocumentURI)
 {
+	TRACE("");
 	return origHttpChannelInternal->GetDocumentURI(aDocumentURI);
 }
 
 NS_IMETHODIMP
 fkHttpChannel::SetDocumentURI(nsIURI *aDocumentURI)
 {
+	TRACE("");
 	return origHttpChannelInternal->SetDocumentURI(aDocumentURI);
 }
 
@@ -545,6 +806,7 @@ fkHttpChannel::GetResponseVersion(PRUint32 *major, PRUint32 *minor)
 NS_IMETHODIMP
 fkHttpChannel::SetCookie(const char *aCookieHeader)
 {
+	TRACE("");
 	return origHttpChannelInternal->SetCookie(aCookieHeader);
 }
 
@@ -562,18 +824,21 @@ fkHttpChannel::GetProxyInfo(nsIProxyInfo **result)
 NS_IMETHODIMP
 fkHttpChannel::GetPriority(PRInt32 *value)
 {
+	TRACE("");
 	return origSupportsPriority->GetPriority(value);
 }
 
 NS_IMETHODIMP
 fkHttpChannel::SetPriority(PRInt32 value)
 {
+	TRACE("");
 	return origSupportsPriority->SetPriority(value);
 }
 
 NS_IMETHODIMP
 fkHttpChannel::AdjustPriority(PRInt32 delta)
 {
+	TRACE("");
 	return origSupportsPriority->AdjustPriority(delta);
 }
 
@@ -598,14 +863,14 @@ NS_IMETHODIMP
 fkHttpChannel::OnStartRequest(nsIRequest *request, nsISupports *ctxt)
 {
 	TRACE("");
-	return origRequestObserver->OnStartRequest(request, ctxt);
+	return origRequestObserver->OnStartRequest(this, ctxt);
 }
 
 NS_IMETHODIMP
 fkHttpChannel::OnStopRequest(nsIRequest *request, nsISupports *ctxt, nsresult status)
 {
 	TRACE("");
-	return origRequestObserver->OnStopRequest(request, ctxt, status);
+	return origRequestObserver->OnStopRequest(this, ctxt, status);
 }
 
 //-----------------------------------------------------------------------------
@@ -618,7 +883,7 @@ fkHttpChannel::OnDataAvailable(nsIRequest *request, nsISupports *ctxt,
                                PRUint32 offset, PRUint32 count)
 {
 	TRACE("");
-	return origStreamListener->OnDataAvailable(request, ctxt, input,
+	return origStreamListener->OnDataAvailable(this, ctxt, input,
 						   offset, count);
 }
 
@@ -630,6 +895,7 @@ NS_IMETHODIMP
 fkHttpChannel::OnTransportStatus(nsITransport *trans, nsresult status,
                                  PRUint64 progress, PRUint64 progressMax)
 {
+	TRACE("");
 	return origTransportEventSink->OnTransportStatus(trans, status, progress,
 							 progressMax);
 } 
@@ -641,12 +907,14 @@ fkHttpChannel::OnTransportStatus(nsITransport *trans, nsresult status,
 NS_IMETHODIMP
 fkHttpChannel::GetCacheToken(nsISupports **token)
 {
+	TRACE("");
 	return origCachingChannel->GetCacheToken(token);
 }
 
 NS_IMETHODIMP
 fkHttpChannel::SetCacheToken(nsISupports *token)
 {
+	TRACE("");
 	return origCachingChannel->SetCacheToken(token);
 }
 
@@ -694,6 +962,7 @@ NS_IMETHODIMP
 fkHttpChannel::ResumeAt(PRUint64 aStartPos,
                         const nsACString& aEntityID)
 {
+	TRACE("");
 	return origResumableChannel->ResumeAt(aStartPos,aEntityID);
 }
 
@@ -739,7 +1008,6 @@ fkHttpChannel::DoAuthRetry(nsAHttpConnection *conn)
 //-----------------------------------------------------------------------------
 // fkHttpChannel::nsIPropertyBag
 //-----------------------------------------------------------------------------
-
 NS_IMETHODIMP
 fkHttpChannel::GetProperty(const nsAString& name, nsIVariant* *_retval)
 {
