@@ -1,8 +1,8 @@
-/*$Id: fkHttpHandler.cpp,v 1.23 2007/03/15 23:34:47 jwrobel Exp $*/
+/*$Id: fkHttpHandler.cpp,v 1.26 2007/07/22 20:05:21 jwrobel Exp $*/
 /* ***** BEGIN LICENSE BLOCK *****
  *  This file is part of Firekeeper.
  *
- *  Copyright (C) 2006 Jan Wrobel <wrobel@blues.ath.cx>
+ *  Copyright (C) 2006, 2007 Jan Wrobel <wrobel@blues.ath.cx>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -53,6 +53,10 @@
 #include "Common.h"
 #include "ActiveRuleSet.h"
 
+nsDataHashtable<nsISupportsHashKey, fkHttpChannel *> *requestHashTable = nsnull;
+nsDataHashtable<nsISupportsHashKey, fkCallbacks *> *callbacksHashTable = nsnull;
+nsDataHashtable<nsISupportsHashKey, fkLoadGroup *> *groupHashTable = nsnull;
+
 
 fkHttpHandler::fkHttpHandler()
 {   
@@ -83,6 +87,20 @@ fkHttpHandler::init(nsIHttpProtocolHandler *origHttpHandler)
 	origObserver = do_QueryInterface(origHttpHandler, &rv);	
 	
 	tracingEnabled = PR_TRUE;
+
+	rv = NS_ERROR_OUT_OF_MEMORY;
+	groupHashTable = new nsDataHashtable<nsISupportsHashKey, fkLoadGroup *>;
+	if (!groupHashTable || NS_FAILED(rv = groupHashTable->Init()))
+		return rv;
+
+	callbacksHashTable = new nsDataHashtable<nsISupportsHashKey, fkCallbacks *>;
+	if (!callbacksHashTable || NS_FAILED(rv = callbacksHashTable->Init()))
+		return rv;
+
+	requestHashTable = new nsDataHashtable<nsISupportsHashKey, fkHttpChannel *>;
+	if (!requestHashTable || NS_FAILED(rv = requestHashTable->Init()))
+		return rv;
+
 	return NS_OK;
 }
 
@@ -176,8 +194,11 @@ fkHttpHandler::NewProxiedChannel(nsIURI *uri,
                                  nsIProxyInfo* givenProxyInfo,
                                  nsIChannel **result)
 {
+	NS_ENSURE_ARG_POINTER(result);
 	nsresult rv;	
 	nsEmbedCString s;
+	nsCOMPtr<nsIChannel> origChannel;
+
 	TRACE("proxy info = %08x", givenProxyInfo);
 	if (!tracingEnabled || fkJudge == nsnull){
 		TRACE("not tracing new channel");
@@ -196,7 +217,7 @@ fkHttpHandler::NewProxiedChannel(nsIURI *uri,
 
 	/*
 	  Check URL after canonicalization eg. http://www.something.com/hehe/
-	  instead of http://www.something.com:80/hehe/hehe/..
+	  instead of http://www.something.com:80/hehe/
 	*/
 	uri->GetSpec(s);
 	const char *url = s.get();
@@ -207,12 +228,14 @@ fkHttpHandler::NewProxiedChannel(nsIURI *uri,
 	if (!connRules)
 		return NS_ERROR_OUT_OF_MEMORY;
 
+
 	list<const Rule *> match = connRules->checkURL(url, strlen(url));
 	list<const Rule *>::iterator it;
 	for(it = match.begin(); it != match.end(); ++it){
 		fkRule *rule_raw = new fkRule(*it);
 		if (!rule_raw){
-			return NS_ERROR_OUT_OF_MEMORY;
+			rv = NS_ERROR_OUT_OF_MEMORY;
+			goto out;
 		}
 		nsCOMPtr<fkIRule> 
 			rule(do_QueryInterface(rule_raw));
@@ -222,40 +245,49 @@ fkHttpHandler::NewProxiedChannel(nsIURI *uri,
 		TRACE("action = %d", action);
 		if (action == sentence->BLOCK){
 			TRACE("aborting request");
-			return NS_ERROR_ABORT;
+			rv = NS_ERROR_ABORT;
+			goto out;
 		}
 		else if (action == sentence->DONT_AUDIT){
 			TRACE("passing request");
-			break;
+			rv = origProxiedHandler->NewProxiedChannel(uri, 
+							      givenProxyInfo,
+							      result);
+			goto out;
 		}
 		/*else (action == sentence->AUDIT) continue with remaining
 		  rules*/
 	}
 
 	/*Create new Mozilla's channel*/
-	nsCOMPtr<nsIChannel> origChannel;
 	rv = origProxiedHandler->NewProxiedChannel(uri, givenProxyInfo, 
 						   getter_AddRefs(origChannel)
 						   );
 	if (NS_FAILED(rv)){
-		TRACE("failed");
-		return rv;
+		goto out;
 	}
 
-	/*Create new Firekeeper's channel that will pass traffic to Mozilla's 
+	/*Create new Firekeeper channel that will pass traffic to Mozilla
 	  channel after checking it*/
 	fkHttpChannel *fkChannel = 
 		new fkHttpChannel(origChannel, sentence, connRules);
-	if (!fkChannel){
+	
+	
+	rv = NS_ERROR_OUT_OF_MEMORY;	
+	if (!fkChannel || NS_FAILED((rv = fkChannel->init()))){
 		*result = NULL;
-		return NS_ERROR_OUT_OF_MEMORY;
+		goto out;
 	}
+	
 	NS_ADDREF(fkChannel);	
-
-	/*TODO: NULL pointer check*/
 	*result = fkChannel;
-
+	requestHashTable->Put(origChannel, fkChannel);
+	
 	return NS_OK;
+	
+ out:
+	delete connRules;
+	return rv;
 }
 
 //-----------------------------------------------------------------------------
@@ -396,6 +428,7 @@ fkHttpHandler::Observe(nsISupports *subject,
                        const char *topic,
                        const PRUnichar *data)
 {
+	TRACE("");
 	if (origObserver)
 		return origObserver->Observe(subject, topic, data);
 	
