@@ -1,4 +1,4 @@
-/*$Id: fkRuleReference.h,v 1.3 2006/08/20 16:39:54 jwrobel Exp $*/
+/*$Id: fkRuleReference.h,v 1.5 2006/11/04 21:09:25 jwrobel Exp $*/
 
 /* ***** BEGIN LICENSE BLOCK *****
  *  This file is part of Firekeeper.
@@ -6,8 +6,9 @@
  *  Copyright (C) 2006 Jan Wrobel <wrobel@blues.ath.cx>
  *
  *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License Version 2 as
- *  published by the Free Software Foundation.
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -33,11 +34,9 @@ public:
 
 
         fkRuleReference(const reference*);
-private:
-	~fkRuleReference();
-	
+	virtual ~fkRuleReference();
+private:	
 	const reference *ref;
-protected:
 
 };
 
