@@ -1,9 +1,9 @@
-/*$Id: fkStreamListener.cpp,v 1.19 2007/02/13 00:11:14 jwrobel Exp $*/
+/*$Id: fkStreamListener.cpp,v 1.22 2007/07/22 20:05:21 jwrobel Exp $*/
 
 /* ***** BEGIN LICENSE BLOCK *****
  *  This file is part of Firekeeper.
  *
- *  Copyright (C) 2006 Jan Wrobel <wrobel@blues.ath.cx>
+ *  Copyright (C) 2006,2007 Jan Wrobel <wrobel@blues.ath.cx>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -63,11 +63,12 @@
 #define NO_FLUSH_POINT_MIN 100
 #define NO_FLUSH_POINT_MAX 200  
 
+#define REQUEST request
 
-
-NS_IMPL_ISUPPORTS3(fkStreamListener, nsIStreamListener,
+NS_IMPL_ISUPPORTS4(fkStreamListener, nsIStreamListener,
 		   nsIRequestObserver, 
-		   nsIHttpHeaderVisitor);
+		   nsIHttpHeaderVisitor,
+		   fkIStreamListenerProxy);
 
 /*
   fkStreamListener constructor
@@ -87,7 +88,8 @@ fkStreamListener::fkStreamListener(nsIStreamListener *origListener,
 	headers_max(HEADERS_INIT_SIZE), offset(0), debugNoHeaders(true),
 	flush_point(0), no_flush_point(0)
 {	
-	TRACE("constructor %08x", this);
+	TRACE("constructor %08x origListener = %08x", 
+	      this, origListener);
 	
 	this->origListener = origListener;
 	this->sentence = sentence;
@@ -118,7 +120,7 @@ fkStreamListener::~fkStreamListener()
 {
 	TRACE("destructor %08x", this);
 	NS_RELEASE(this->channel);
-		
+	
 	if (body != nsnull)
 		nsMemory::Free(body);
 	if (headers != nsnull)
@@ -238,22 +240,17 @@ fkStreamListener::checkBody()
 }
 
 
-
 /*
   Check body fragment and pass it to the original listener if it is ok.
   @buffer
          body fragment to check 
  */
 NS_IMETHODIMP
-fkStreamListener::do_OnDataAvailable(nsIRequest* request,
-				     nsISupports *aContext)
+fkStreamListener::do_OnDataAvailable(nsIRequest *request, nsISupports *aContext)
 {
-
 	nsresult rv;
-	
 	TRACE("calling checkBody len =%d", body_len);
 	rv = checkBody();
-
 	if (NS_FAILED(rv)){
 		TRACE("check body failed");
 		return rv;
@@ -288,7 +285,7 @@ fkStreamListener::do_OnDataAvailable(nsIRequest* request,
 		body_len = 0;
 	}
 	
-	rv = origListener->OnDataAvailable(request, aContext, 
+	rv = origListener->OnDataAvailable(getRequestProxy(request), aContext, 
 					   convertedStream, 
 					   offset, data_len);
 	offset += data_len;;
@@ -300,15 +297,13 @@ fkStreamListener::do_OnDataAvailable(nsIRequest* request,
     if it is ok. Free body buffer.
  */
 NS_IMETHODIMP
-fkStreamListener::do_LastOnDataAvailable(nsIRequest *request, 
-					 nsISupports* acontext)
+fkStreamListener::do_LastOnDataAvailable(nsIRequest *request, nsISupports* acontext)
 {
 	nsresult rv;
 	FK_ASSERT(body != nsnull);
 	TRACE("calling checkBody");
 	
 	rv = checkBody();
-	TRACE("ok");
 	if (NS_FAILED(rv))
 		return rv;
 
@@ -321,17 +316,32 @@ fkStreamListener::do_LastOnDataAvailable(nsIRequest *request,
 		return rv;
 	
 	body = nsnull;
+	
 	nsCOMPtr<nsIInputStream> convertedStream = 
 		do_QueryInterface(convertedStreamSup, &rv);
 	if (NS_FAILED(rv))
 		return rv;
 	
 	TRACE("OK offset = %d len %d", offset, body_len);
-	return origListener->OnDataAvailable(request, acontext, 
+	return origListener->OnDataAvailable(getRequestProxy(request), acontext, 
 					     convertedStream, offset, 
 					     body_len);
 }
 
+//-----------------------------------------------------------------------------
+// fkStreamListener::fkIStreamListenerProxy
+//-----------------------------------------------------------------------------
+NS_IMETHODIMP 
+fkStreamListener::GetOrigStreamListener(nsIStreamListener **aStreamListener)
+{
+	TRACE("fkStreamListener::fkIStreamListenerProxy");
+	NS_ENSURE_ARG_POINTER(aStreamListener);		
+	FK_ASSERT(origListener);
+	
+	NS_IF_ADDREF(*aStreamListener = origListener);
+	return NS_OK;
+}
+
 //-----------------------------------------------------------------------------
 // fkStreamListener::nsIStreamListener
 //-----------------------------------------------------------------------------
@@ -340,7 +350,8 @@ fkStreamListener::OnDataAvailable(nsIRequest *request, nsISupports *ctxt,
 				  nsIInputStream *input,
 				  PRUint32 off, PRUint32 count)
 {	
-	TRACE("got data count = %d offset = %d", count, off);
+	TRACE("got data count = %d offset = %d request = %08x origchannel = %08x ctx = %08x", 
+	      count, off, request, &*channel, ctxt);
 	FK_ASSERT(debugNoHeaders == false);
 	
 	int action;
@@ -351,7 +362,7 @@ fkStreamListener::OnDataAvailable(nsIRequest *request, nsISupports *ctxt,
 	/*Whitelisted. Don't check it*/
 	if (action == sentence->DONT_AUDIT){		
 		TRACE("whitelisted");
-		return origListener->OnDataAvailable(request, ctxt, input, 
+		return origListener->OnDataAvailable(getRequestProxy(request), ctxt, input, 
 						     off, count);
 	}
 	FK_ASSERT(action != sentence->BLOCK);
@@ -391,13 +402,14 @@ fkStreamListener::OnDataAvailable(nsIRequest *request, nsISupports *ctxt,
 // fkStreamListener::nsIRequestObserver
 //-----------------------------------------------------------------------------
 
-/*
+/*request = 095b6fc4 origchannel = 095a4d08 proxy 095b73c8
   Request started. Get and check headers.
  */
 NS_IMETHODIMP
 fkStreamListener::OnStartRequest(nsIRequest *request, nsISupports* acontext)
 {
-	TRACE("Request started %08x", this);
+	TRACE("Request started %08x request = %08x origchannel = %08x ctx = %08x ", this, 
+	      request, channel, acontext);
 	nsresult rv = channel->VisitResponseHeaders(this);
 	if (NS_FAILED(rv)){
 		TRACE("Visit response headers failed");
@@ -436,32 +448,37 @@ fkStreamListener::OnStartRequest(nsIRequest *request, nsISupports* acontext)
 		}
 	}
 
-	return origListener->OnStartRequest(request, acontext);
+	TRACE("call");
+	rv = origListener->OnStartRequest(getRequestProxy(request), acontext);
+	TRACE("returned");
+	return rv;
 }
 
 /*
   Request stopped. Check what is left in body buffer
 */
 NS_IMETHODIMP
-fkStreamListener::OnStopRequest(nsIRequest *request, nsISupports* acontext,
+fkStreamListener::OnStopRequest(nsIRequest *request, nsISupports *acontext,
 				nsresult astatus)
 {
-	TRACE("Request stopped %08x status = %d", this, astatus);
+	TRACE("Request stopped %08x status = %d request = %08x origchannel = %08x ctx = %08x", 
+	      this, astatus, request, channel, acontext);
 	nsresult rv;
 	int action;
 	sentence->GetAction(&action);
 	
 	if (body_len && action != sentence->DONT_AUDIT && 
 	    action != sentence->BLOCK){
+		TRACE("passing remaining body fragment");
 		no_flush_point = 0;
-		rv = do_OnDataAvailable(request, acontext);	
+		rv = do_LastOnDataAvailable(request, acontext);	
 		if (NS_FAILED(rv)){
 			TRACE("do_OnDataAvailable failed %d", rv);
 			astatus = rv;
 		}
 	}
 	
-	return origListener->OnStopRequest(request, acontext, astatus);
+	return origListener->OnStopRequest(getRequestProxy(request), acontext, astatus);
 
 }
 
