@@ -1,4 +1,4 @@
-/*$Id: RulesParser.cpp,v 1.22 2007/02/13 00:11:14 jwrobel Exp $*/
+/*$Id: RulesParser.cpp,v 1.23 2007/03/23 11:49:03 jwrobel Exp $*/
 /* ***** BEGIN LICENSE BLOCK *****
  *  This file is part of Firekeeper.
  *
@@ -308,7 +308,7 @@ bool RulesParser::parseRE(char *data, const char *name,
     else if(!(*re == delimit))
 	    goto syntax;
 
-    TRACE("DELIMETER %c", *re);
+    TRACE("delimeter %c", *re);
     /* find ending delimiter, trim delimit chars */
     opts = strrchr(re, delimit);
     if(!((opts - re) > 1)) /* empty regex(m||) or missing delim not OK */
@@ -757,6 +757,7 @@ char *RulesParser::parseOptions(char *options, Rule *rule)
 	static char errbuf[ERRBUFLEN];
 	RE re;
 	PatternMatchData cont;
+	bool correct_cont;
 	
 	TRACE("PARSING OPTIONS %s", options);
 	rule->msg = parseOption(msgRE, options);
@@ -802,36 +803,33 @@ char *RulesParser::parseOptions(char *options, Rule *rule)
 	
 	while ((res = parseContentOption(urlContentRE, options, 
 					 &cont.nocase))){
-		
-		if (!parseContent(res, "url_content:", &cont, errbuf, 
-				  ERRBUFLEN)){
-			free(res);
+		correct_cont = parseContent(res, "url_content:", &cont, errbuf, 
+					    ERRBUFLEN);
+		free(res);
+		if (!correct_cont)
 			return errbuf;
-		}
 		cont.parent = rule;
 		rule->url_content.push_back(cont);
 	}
 	
 	while ((res = parseContentOption(headersContentRE, options, 
 					 &cont.nocase))){
-		
-		if (!parseContent(res, "headers_content:", &cont, errbuf, 
-				  ERRBUFLEN)){
-			free(res);
+		correct_cont = parseContent(res, "headers_content:", &cont, errbuf, 
+					    ERRBUFLEN);
+		free(res);		
+		if (!correct_cont)
 			return errbuf;
-		}
 		cont.parent = rule;
 		rule->headers_content.push_back(cont);
 	}
 	
 	while ((res = parseContentOption(bodyContentRE, options, 
 					 &cont.nocase))){
-		
-		if (!parseContent(res, "body_content:", &cont, errbuf, 
-				  ERRBUFLEN)){
-			free(res);
+		correct_cont = parseContent(res, "body_content:", &cont, errbuf, 
+					    ERRBUFLEN);
+		free(res);
+		if (!correct_cont)
 			return errbuf;
-		}
 		cont.parent = rule;
 		rule->body_content.push_back(cont);
 	}
@@ -1021,7 +1019,7 @@ void RulesParser::parseRules(Rules *rules, const char *input, char **error)
 
 	while(!done){	
 		linenr++;	
-		for(;input[idx] && input[idx] != '\n' && idx < RULEBUFLEN; 
+		for(;idx < RULEBUFLEN && input[idx] && input[idx] != '\n'; 
 		    idx++)
 			buf[idx] = input[idx];
 		
@@ -1037,8 +1035,7 @@ void RulesParser::parseRules(Rules *rules, const char *input, char **error)
 		input += idx + 1;
 		idx = 0;
 	}
-	
-	       
+
 	if (strlen(errmsg) == 0)
 		*error = NULL;
 }
